// CheckACLine.cpp : Defines the entry point for the application.
//

#include "stdafx.h"

#include "PowerWatch.h"
#include "Util.h"

#define	CHECKACLINE_APP_MUTEX			_T("CheckACLine Mutex")
#define	CHECKACLINE_STOP_EVENT			_T("CheckACLine Stop Event")
#define	CHECKACLINE_STATUSCHANGE_EVENT	_T("CheckACLine StatusChange Event")

#define CHECKACLINE_SETTING_REGKEY		_T("Software\\CheckACLine")
#define	CHECKACLINE_CONNECT_PATH		_T("connect_cmd")
#define	CHECKACLINE_CONNECT_ARG			_T("connect_arg")
#define	CHECKACLINE_DISCONNECT_PATH		_T("disconnect_cmd")
#define CHECKACLINE_DISCONNECT_ARG		_T("disconnect_arg")

#define CHECKACLINE_PERIOD_TIME			(5*1000)

#define ACLINE_STATUS_UNKNOWN			0
#define ACLINE_STATUS_ONLINE			1
#define ACLINE_STATUS_OFFLINE			2

static
BOOL
ExecUserAction(
	LPTSTR	pCmdValueName,
	LPTSTR	pArgValueName
)
{
	BOOL	bRet;
	LONG	lResult;
	HKEY	hKey = NULL;

	bRet = FALSE;	// default return value

	lResult = RegOpenKeyEx(HKEY_CURRENT_USER, CHECKACLINE_SETTING_REGKEY, 0, KEY_ALL_ACCESS, &hKey);

	if (lResult == ERROR_SUCCESS) {
		TCHAR	szPath[MAX_PATH+1];
		TCHAR	szArg[MAX_PATH+1];
		DWORD	dwType = 0;
		DWORD	dwSize = 0;

		memset(szPath, 0, sizeof(szPath));
		dwSize = sizeof(szPath);

		lResult = RegQueryValueEx(hKey, pCmdValueName, NULL, &dwType, (LPBYTE)szPath, &dwSize);
		if (lResult == ERROR_SUCCESS && _tcslen(szPath) != 0) {
			BOOL				bCreateSucc = FALSE;
			PROCESS_INFORMATION pi;

			memset(&pi, 0, sizeof(pi));

			memset(szArg, 0, sizeof(szArg));
			dwSize = sizeof(szArg);
			lResult = RegQueryValueEx(hKey, pArgValueName, NULL, &dwType, (LPBYTE)szArg, &dwSize);
			if (lResult == ERROR_SUCCESS && _tcslen(szArg) != 0) {
				bCreateSucc = CreateProcess(szPath, szArg, NULL, NULL, FALSE, 0, NULL, NULL, NULL, &pi);
			}
			else {
				bCreateSucc = CreateProcess(szPath, NULL, NULL, NULL, FALSE, 0, NULL, NULL, NULL, &pi);
			}

			if (bCreateSucc) {
				CloseHandle(pi.hThread);
				CloseHandle(pi.hProcess);
			}

			bRet = TRUE;
		}

		RegCloseKey(hKey);
		hKey = NULL;
	}

	return bRet;
}

static
BOOL
GetACLineStatus(
	LPDWORD		pdwStatus
)
{
	BOOL					bRet;
	SYSTEM_POWER_STATUS_EX2 SystemPowerStatusEx2;
	DWORD					dwRet;

	bRet = FALSE;

	if (pdwStatus == NULL) {
		return FALSE;
	}

	dwRet = GetSystemPowerStatusEx2(
			&SystemPowerStatusEx2,
			sizeof(SystemPowerStatusEx2),
			FALSE
			);

	if (dwRet) {
		switch(SystemPowerStatusEx2.ACLineStatus) {
		case AC_LINE_OFFLINE:
			*pdwStatus = ACLINE_STATUS_OFFLINE;
			break;
		case AC_LINE_ONLINE:
			*pdwStatus = ACLINE_STATUS_ONLINE;
			break;
		default:
			*pdwStatus = ACLINE_STATUS_UNKNOWN;
			break;
		}

		bRet = TRUE;
	}

	return bRet;
}

static
void
WINAPI
NotifyPowerStatusChange(
	void*	pParam
)
{
	HANDLE	hNotifyEvent;
	
	hNotifyEvent = (HANDLE)pParam;
	if (hNotifyEvent != NULL) {
		SetEvent(hNotifyEvent);
	}
}


int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	HANDLE		hMutex			= NULL;
	HANDLE		hStopEvent		= NULL;
	HANDLE		hNotifyEvent	= NULL;

	// dN`FbN
	hMutex = CreateMutex(NULL, TRUE, CHECKACLINE_APP_MUTEX);
	if (hMutex != NULL) {
		if (GetLastError() == ERROR_ALREADY_EXISTS) {

			INT		iRet;

			iRet = Util_MessageBox(NULL, hInstance, IDS_CHECKACLINE_CONFIRM, IDS_CHECKACLINE_CAPTION, MB_ICONQUESTION|MB_YESNO);
			if (iRet == IDYES) {
				hStopEvent = OpenEvent(EVENT_ALL_ACCESS, FALSE, CHECKACLINE_STOP_EVENT);
				if (hStopEvent != NULL) {
					SetEvent(hStopEvent);
					CloseHandle(hStopEvent);
					hStopEvent = NULL;
				}
			}

			CloseHandle(hMutex);
			hMutex = NULL;
			return FALSE;
		}
	}
	else {
		return FALSE;
	}
	//


	hStopEvent		= CreateEvent(NULL, FALSE, FALSE, CHECKACLINE_STOP_EVENT);
	hNotifyEvent	= CreateEvent(NULL, FALSE, FALSE, CHECKACLINE_STATUSCHANGE_EVENT);

	if (hStopEvent != NULL && hNotifyEvent != NULL) {
		BOOL			bWatchSucc;
		CPowerWatch		powerWatch;

		bWatchSucc = powerWatch.StartWatch(NotifyPowerStatusChange, (void*)hNotifyEvent);
		if (bWatchSucc) {
			HANDLE	pEvents[2];
			BOOL	bStopRequest	= FALSE;
			DWORD	dwPrevStatus	= ACLINE_STATUS_UNKNOWN;
			DWORD	dwStatus		= ACLINE_STATUS_UNKNOWN;

			pEvents[0] = hStopEvent;
			pEvents[1] = hNotifyEvent;

			GetACLineStatus(&dwPrevStatus);

			while (!bStopRequest) {
				DWORD	dwObject;

				dwObject = WaitForMultipleObjects(2, pEvents, FALSE, INFINITE);
				switch(dwObject) {
				case WAIT_OBJECT_0:
					// Iʒmꍇ
					bStopRequest = TRUE;
					break;

				case WAIT_OBJECT_0 + 1:
					// ACLineStatus̎擾
					BOOL	bGetSucc;

					bGetSucc = GetACLineStatus(&dwStatus);
					if (bGetSucc) {
						if (dwPrevStatus == ACLINE_STATUS_OFFLINE) {
							if (dwStatus == ACLINE_STATUS_ONLINE) {
								// ACIt -> ACI
								ExecUserAction(CHECKACLINE_CONNECT_PATH, CHECKACLINE_CONNECT_ARG);
							}
						}
						else if (dwPrevStatus == ACLINE_STATUS_ONLINE) {
							if (dwStatus == ACLINE_STATUS_OFFLINE) {
								// ACI -> ACIt
								ExecUserAction(CHECKACLINE_DISCONNECT_PATH, CHECKACLINE_DISCONNECT_ARG);
							}
						}

						dwPrevStatus = dwStatus;
					}
					break;
				}
			}

			powerWatch.StopWatch();
		}

	}

	if (hStopEvent != NULL) {
		CloseHandle(hStopEvent);
		hStopEvent = NULL;
	}
	if (hNotifyEvent != NULL) {
		CloseHandle(hNotifyEvent);
		hNotifyEvent = NULL;
	}

	ReleaseMutex(hMutex);
	CloseHandle(hMutex);
	hMutex = NULL;
	return 0;
}

